import numpy
import numpy as np
import requests
from bs4 import BeautifulSoup
import openai
import os
import spacy
import random
import nltk
import re
from unidecode import unidecode
import discord
from discord.ext import commands
import asyncio
import pandas as pd
from sklearn.exceptions import UndefinedMetricWarning
from sklearn.feature_extraction.text import CountVectorizer
from sklearn.model_selection import train_test_split
from sklearn.naive_bayes import MultinomialNB
from sklearn.metrics import classification_report

import warnings

warnings.filterwarnings("ignore", category=UndefinedMetricWarning)

data = pd.read_csv('dane.csv', sep=',')
data['tekst'] = data['tekst'].str.lower()

X = data['tekst'].values
y = data['klasa'].values

vectorizer = CountVectorizer()
X = vectorizer.fit_transform(X)

X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, random_state=2137)

model = MultinomialNB()
model.fit(X_train, y_train)

y_pred = model.predict(X_test)


def znajdz_klase(tekst):
    nowe_dane = vectorizer.transform([tekst])
    predykcja = model.predict(nowe_dane)
    return predykcja[0]


openai.api_key = os.environ["OPENAI_API_KEY"]
URL = "https://www.pk.edu.pl/index.php?lang=pl"
random.seed()

stop_words = []
with open("stopwords", "r") as f:
    for line in f:
        stop_words.append(line.strip())


def pobierz_opis(link):
    response = requests.get(link)
    if response.status_code != 200:
        return ""

    soup = BeautifulSoup(response.content, "html.parser")
    opis = soup.find("div", itemprop="articleBody")

    if opis:
        tresc = opis.get_text().strip()

        if len(tresc) > 3000:
            tresc = tresc.lower()
            tresc = re.sub(r'[^\w\s]', '', tresc)
            tresc = re.sub(r"http\S+", "", tresc)
            tokens = tresc.split(' ')
            filtered_text = [word for word in tokens if not word in stop_words]
            tresc = " ".join(filtered_text)

        if len(tresc) > 3000:
            tresc = tresc[:3000]

        prompt = f"Podsumuj artykuł w jak najbardziej zwięzły sposób, ale nie przekraczający 100 słów (lub 300 tokenów)." \
                 f" Odpowiedź wygeneruj w języku polskim. Treść artykułu: {tresc}"
        response = openai.Completion.create(engine="text-davinci-003", prompt=prompt, max_tokens=500, n=1, stop=None,
                                            temperature=0.7)
        podsumowanie = response.choices[0].text.strip()

        return podsumowanie
    else:
        return ""


def pobierz_newsy(liczba_newsy=5):
    response = requests.get(URL)
    if response.status_code != 200:
        return "Wystąpił problem z pobieraniem informacji z witryny Politechniki Krakowskiej."

    soup = BeautifulSoup(response.content, "html.parser")
    newsy = soup.find_all("div", class_="category-title")

    wynik = []
    for idx, news in enumerate(newsy[:liczba_newsy]):
        tytul = news.find("a", class_="mod-articles-category-title").get_text().strip()
        link = "https://www.pk.edu.pl" + news.find("a", class_="mod-articles-category-title")["href"]
        wynik.append(f"{tytul}\nLink: {link}")

    return wynik


def pobierz_kierunki():
    url = "https://rekrutacja.pk.edu.pl/oferta-edukacyjna/"
    response = requests.get(url)
    if response.status_code != 200:
        return "Wystąpił problem z pobieraniem informacji z witryny Politechniki Krakowskiej."

    soup = BeautifulSoup(response.content, "html.parser")

    wynik = ""
    for a_tag in soup.find_all('h2', {'class': 'et_pb_module_header'}):
        text = a_tag.get_text()
        text = re.sub(r'\s+', ' ', text)
        link = a_tag.find("a")["href"]
        text = f"{text}(<{link}>)\n"
        wynik += text

    return wynik


def remove_diacritics(text):
    return unidecode(text)


def cos_sim(a, b):
    return np.dot(a, b) / (np.linalg.norm(a) * np.linalg.norm(b))


def get_intent(pytanie):
    return znajdz_klase(pytanie)


def odpowiedz_na_pytanie(pytanie):
    pytanie = pytanie.lower()
    pytanie = remove_diacritics(pytanie)
    intent = get_intent(pytanie)

    if intent == "aktualnosci":
        numer_newsa = random.randint(0, 4)
        news = pobierz_newsy(5)[numer_newsa]
        linki_newsy = pobierz_linki_newsy()
        opis = pobierz_opis(linki_newsy[numer_newsa])
        return "Oto najnowsze informacje z Politechniki Krakowskiej:\n" + news + "\n" + opis
    if intent == "powitanie":
        return "Witaj! W czym mogę pomóc?"
    if intent == "kierunki":
        return "Oto kierunki studiów:\n" + pobierz_kierunki()
    else:
        return "Nie rozumiem pytania. Poproś o informacje o Politechnice Krakowskiej lub o najnowsze newsy."


def pobierz_linki_newsy(liczba_newsy=5):
    response = requests.get(URL)
    soup = BeautifulSoup(response.content, "html.parser")
    newsy = soup.find_all("div", class_="category-title")
    linki_newsy = []

    for news in newsy[:liczba_newsy]:
        link = "https://www.pk.edu.pl" + news.find("a", class_="mod-articles-category-title")["href"]
        linki_newsy.append(link)

    return linki_newsy


intents = discord.Intents.default()
intents.message_content = True
bot = commands.Bot(command_prefix="!", intents=intents)


@bot.event
async def on_ready():
    print(f"{bot.user.name} połączony z Discord!")


@bot.command(name="pytanie", help="Zadaj pytanie dotyczące Politechniki Krakowskiej lub najnowszych newsów.")
async def pytanie(ctx, *, arg):
    await ctx.send("Generowanie odpowiedzi...");
    odpowiedz = odpowiedz_na_pytanie(arg)
    bloki_odpowiedzi = podziel_na_bloki(odpowiedz)

    for blok in bloki_odpowiedzi:
        await ctx.send(blok)


@bot.command()
async def test(ctx, arg):
    await ctx.send(arg)


async def run_bot():
    await bot.start(os.environ["DISCORD_TOKEN"])


def podziel_na_bloki(tekst, limit=1900):
    bloki = []
    while len(tekst) > limit:
        index = tekst[:limit].rfind(" ")
        bloki.append(tekst[:index])
        tekst = tekst[index + 1:]

    bloki.append(tekst)
    return bloki


if __name__ == "__main__":
    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)
    try:
        loop.run_until_complete(run_bot())
    finally:
        loop.close()
